%% NNL Transfer Matrix model
%   This is a matlab cell script that calculates the R,T and A of multilayer stacks
%   To use, all you need to do is press 'Ctrl+Alt+Enter' through each of the cells
%   This initialises the nnl_root.m file - this is a separate file containing the functions necessary for the simulation and should be stored in the same folder as this cell script.
%   The next cell initialises the spectra and materials - the next cell
%   along (you need to make sure you've got the 'reference_data' folder in
%   the working directory.
%   The next cell of the script sets the solar-cell parameters - further details on material parameters can be found in the nnl_root.m
%   We can now calculate a tandem solar cell, either statically, or by scanning through cell thicknesses and light trapping strengths.
%   The final cell prints out all the lovely graphs...
%   Good luck! happy to help if you're stuck
%
%   - Nij

%   Dr Niraj Lal | w: nirajlal.org | e: niraj@nirajlal.org, 2017

%% Clear all
clear all;          % clear all variables

%% Initialise root
import nnl_root.*   % import functions from nnl_root - all of the below functions are contained in the nnl_root.m file
nnl_import_check;   % print a message if everything imported ok

%% Initialise spectra and materials

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
wl = 400:2:1200;                    % set wavelength range and grain here
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nnl_initialise_spectra(wl);         % load AM1.5G spectra and interpolate to the above wavelength range, set in workspace
nnl_import_materials(wl);
%% Transfer matrix calculator 2 complete
clear TM_complete, clear currentTM, clear TMinput;  
clear rr, clear tt, clear RR, clear TT, clear AA;

%%%%%%%%%%%%% Input light %%%%%%%%%%%%%%%
TMinput.spectra = light;  %light includes wl and sunshine
TMinput.angledeg = 0; % in degrees
TMinput.light = light;
%%%%%%%%%%%%% structure %%%%%%%%%%%%%%%

%     Test structure
    TMinput.superstrate = air;
    TMinput.substrate = air;
    TMinput.activelayer = 2; % for solar cell calculation
    TMinput.numberoflayers = 4;
    
    TMinput.layer(1).nk = PMMA.nk;
    TMinput.thickness(1) = 80; %in nm
    TMinput.layer(2).nk = Perovskite_Loper.nk;
    TMinput.thickness(2) = 600; %in nm
    TMinput.layer(3).nk = TiO2_anatase.nk;
    TMinput.thickness(3) = 50; %in nm
    TMinput.layer(4).nk = FTO_Wenger.nk;
    TMinput.thickness(4) = 400; %in nm



%%%%%% Single structures %%%%
%    TMinput.superstrate = air;
%    TMinput.substrate = air;
    
%    TMinput.activelayer = 1; % for cell calculation
% 
%     
%       TMinput.numberoflayers = 1;
%      TMinput.layer(1).nk = SiO2.nk;
%      TMinput.thickness(1) =200; %in nm

    % 
%%%%%%%%%%%%% McGehee Test Organic Solar Cell %%%%%%%%%%%%%%%%%%
%     TMinput.numberoflayers = 5;
%     TMinput.layer(1).nk = ITO_MG.nk;
%     TMinput.thickness(1) = 110; %in nm
%     TMinput.layer(2).nk = PEDOT_MG.nk;
%     TMinput.thickness(2) = 35; %in nm
%     TMinput.layer(3).nk = P3HTPCBM_MG.nk;
%     TMinput.thickness(3) = 220; %in nm
%     TMinput.layer(4).nk = Ca_MG.nk;
%     TMinput.thickness(4) = 7; %in nm
%     TMinput.layer(5).nk = Al_MG.nk;
%     TMinput.thickness(5) = 200; %in nm
%     

%%%%%%%%%%%%% Monash planar perovskite solar cell %%%%%%%%%%%%%%%%%%
%     TMinput.numberoflayers = 6;
%     TMinput.layer(1).nk = Al2O3_Malitson.nk;
%     TMinput.thickness(1) = 85; %in nm
%     TMinput.layer(2).nk = FTO_Wenger.nk;
%     TMinput.thickness(2) = 600; %in nm
%     TMinput.layer(3).nk = TiO2_anatase.nk;
%     TMinput.thickness(3) = 30; %in nm
%     TMinput.layer(4).nk = Perovskite_Loper.nk;
%     TMinput.thickness(4) = 450; %in nm
%     TMinput.layer(5).nk = Spiro.nk;
%     TMinput.thickness(5) = 100; %in nm
%     TMinput.layer(6).nk = Au_JC.nk;
%     TMinput.thickness(6) = 80; %in nm
    

%%%%%%%%%%%%% Monash IBC perovskite solar cell %%%%%%%%%%%%%%%%%%

% %%%% With Contact %%%%%%%
%     TMinput.superstrate = air;
%     TMinput.substrate = air;
%     TMinput.activelayer = 2; % for cell calculation
%     TMinput.numberoflayers = 7;
% 
%     TMinput.layer(1).nk = PMMA.nk;
%     TMinput.thickness(1) = 80; %in nm
%     TMinput.layer(2).nk = Perovskite_Loper.nk;
%     TMinput.thickness(2) = 370; %in nm
%     TMinput.layer(3).nk = Ni_JC.nk;
%     TMinput.thickness(3) = 50; %in nm
%     TMinput.layer(4).nk = Al_Rakic.nk;
%     TMinput.thickness(4) = 30; %in nm
%     TMinput.layer(5).nk = Al2O3_Malitson.nk;
%     TMinput.thickness(5) = 150; %in nm
%     TMinput.layer(6).nk = TiO2_anatase.nk;
%     TMinput.thickness(6) = 50; %in nm
%     TMinput.layer(7).nk = FTO_Wenger.nk;
%     TMinput.thickness(7) = 400; %in nm

%%% Without Contact %%%%%%%

%     TMinput.superstrate = air;
%     TMinput.substrate = air;
%     TMinput.activelayer = 2; % for solar cell calculation
%     TMinput.numberoflayers = 4;
%     
%     TMinput.layer(1).nk = PMMA.nk;
%     TMinput.thickness(1) = 80; %in nm
%     TMinput.layer(2).nk = Perovskite_Loper.nk;
%     TMinput.thickness(2) = 600; %in nm
%     TMinput.layer(3).nk = TiO2_anatase.nk;
%     TMinput.thickness(3) = 50; %in nm
%     TMinput.layer(4).nk = FTO_Wenger.nk;
%     TMinput.thickness(4) = 400; %in nm
    

% %%%%% Linton %%%%%%%
%     TMinput.superstrate = air;
%     TMinput.substrate = glass;
%     TMinput.activelayer = 2; % for cell calculation
%     TMinput.numberoflayers = 4;
% 
% %     TMinput.layer(1).nk = PMMA.nk;
% %     TMinput.thickness(1) = 0; %in nm
% %     TMinput.layer(2).nk = Perovskite_Loper.nk;
% %     TMinput.thickness(2) = 350; %in nm
% %     TMinput.layer(3).nk = glass.nk;
% %     TMinput.thickness(3) = 50; %in nm
% %     TMinput.layer(4).nk = Al_Rakic.nk;
% %     TMinput.thickness(4) = 100; %in nm
% 
%     TMinput.layer(1).nk = PMMA.nk;
%     TMinput.thickness(1) = 0; %in nm
%     TMinput.layer(2).nk = Perovskite_Loper.nk;
%     TMinput.thickness(2) = 280; %in nm
%     TMinput.layer(3).nk = glass.nk;
%     TMinput.thickness(3) = 0; %in nm
%     TMinput.layer(4).nk = Au_JC.nk;
%     TMinput.thickness(4) = 70; %in nm


%%%%%%%%%%%%%%%%%%%%%%%

% Bragg stacks
% % %%%% Long-pass Bragg stack %%%%
% TMinput.superstrate = air;
% TMinput.substrate = glass;
% TMinput.materialL = SiO2;
% TMinput.materialH = TiO2;

%     % One stack %
%     TMinput.lambda0 = 600;  % needs to be within wl range at the moment
%     TMinput.numHLblocks = 11;
%     TMinput = Create_LongPass_Bragg(TMinput,wl); %Mat1, Mat2, lambda0, numHLblocks needed to create the Bragg stack
    %%%%%%%%%%%%%
    
    % Two stacks %
%     TMinput.lambda01 = 460;  
%     TMinput.numHLblocks1 = 8;
%     TMinput.lambda02 = 600;  % needs to be within wl range at the moment
%     TMinput.numHLblocks2 = 8; % If this is zero - then you need to set lambda02 to be the same as lambda01
%     TMinput = Create_LongPass_Bragg2(TMinput,wl); %Mat1, Mat2, lambda0, numHLblocks needed to create the Bragg stack
% %     %%%%%%%%%%%%%%
    
%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Short-pass Bragg stack %%%%
% TMinput.superstrate = air;
% TMinput.substrate = glass;
% TMinput.materialL = SiO2;
% TMinput.materialH = TiO2;

% One stack %

%     TMinput.lambda0 = 950;  % needs to be within wl range at the moment
%     TMinput.numHLblocks = 6;
%     TMinput = Create_ShortPass_Bragg(TMinput,wl); %Mat1, Mat2, lambda0, numHLblocks needed to create the Bragg stack
    %%%%%%%%%%%%%    


  %  Two stacks %
%     TMinput.lambda01 = 950;  
%     TMinput.numHLblocks1 = 6;
%     TMinput.lambda02 = 800;  % needs to be within wl range at the moment
%     TMinput.numHLblocks2 = 6; % If this is zero (ie. one stack) - then you need to set lambda02 to be the same as lambda01
%     TMinput = Create_ShortPass_Bragg2(TMinput,wl); %Mat1, Mat2, lambda0, numHLblocks needed to create the Bragg stack
%     %%%%%%%%%%%%
    
%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% Initialisation %%%%%%%%
TM.rr = zeros(1,length(wl));
TM.tt = zeros(1,length(wl));
TM.RR = zeros(1,length(wl));
TM.TT = zeros(1,length(wl));
TM.AA = zeros(1,length(wl));
TM.wl = light.wl;

TE.rr = zeros(1,length(wl));
TE.tt = zeros(1,length(wl));
TE.RR = zeros(1,length(wl));
TE.TT = zeros(1,length(wl));
TE.AA = zeros(1,length(wl));
TE.wl = light.wl;

mixed.rr = zeros(1,length(wl));
mixed.tt = zeros(1,length(wl));
mixed.RR = zeros(1,length(wl));
mixed.TT = zeros(1,length(wl));
mixed.AA = zeros(1,length(wl));
mixed.wl = light.wl;


%%%% Calculation %%%%%%%%
for ii= 1:1:length(wl)
    wavelength = wl(ii);
    currentTM = transfermatrixcomplete(TMinput,wavelength);    
    TM.rr(ii) = currentTM.TM.rr;
    TM.tt(ii) = currentTM.TM.tt;
    TM.RR(ii) = currentTM.TM.RR;
    TM.TT(ii) = currentTM.TM.TT;
    TM.AA(ii) = currentTM.TM.AA;
    
    TE.rr(ii) = currentTM.TE.rr;
    TE.tt(ii) = currentTM.TE.tt;
    TE.RR(ii) = currentTM.TE.RR;
    TE.TT(ii) = currentTM.TE.TT;
    TE.AA(ii) = currentTM.TE.AA;

    mixed.rr(ii) = currentTM.mixed.rr;
    mixed.tt(ii) = currentTM.mixed.tt;
    mixed.RR(ii) = currentTM.mixed.RR;
    mixed.TT(ii) = currentTM.mixed.TT;
    mixed.AA(ii) = currentTM.mixed.AA;

end

%%% Populate results %%%
TMoutput.TM = TM;
TMoutput.TE = TE;
TMoutput.mixed = mixed;

% Plot TM results
figure;
GraphRTA = TMoutput.TE; %Choose here either TM, TE or mixed
%plot(GraphRTA.wl, GraphRTA.RR, GraphRTA.wl, GraphRTA.TT, GraphRTA.wl, GraphRTA.AA);
plot(GraphRTA.wl, GraphRTA.RR, GraphRTA.wl, GraphRTA.TT, GraphRTA.wl, GraphRTA.AA);%, GraphRTA.wl, GraphRTA.RR + GraphRTA.TT + GraphRTA.AA);
makegraphnice();
xlabel('Wavelength (nm)'), ylabel('R,T,A');
xlim([min(wl) max(wl)]); 
ylim([0 1]);
%hleg1 = legend('R');
%set(hleg1,'Location','NorthWest')
%set(hleg1,'Interpreter','none')
%set(plot(all),'LineWidth',1)
%set(gcf,'Color',[1 1 1]), title('Tandem efficiency vs. intermediate reflector strength','FontWeight','bold')


% TM Field

%%%%%%%%%%%%% Input x-positions and wavelengths to calculate field over %%%%%%%%%%%%%%
TMinput.sunphotons = light.sunshine.n_of_photons;
TMinput.sunspectra = light.sunshine.spectra; %AM1.5G
TMinput.xgrain = 1; % in nm
TMinput.xpos=TMinput.xgrain/2:TMinput.xgrain:sum(TMinput.thickness);
TMinput.fieldwls = [400 600 800];

%TM_celloutput.TM.RR = zeros(1,length(wl));
TM_celloutput.TE.RR = zeros(1,length(wl));
%TM_celloutput.mixed.RR = zeros(1,length(wl));
%TM_celloutput.TM.TT = zeros(1,length(wl));
TM_celloutput.TE.TT = zeros(1,length(wl));
%TM_celloutput.mixed.TT = zeros(1,length(wl));
%TM_celloutput.TM.AA = zeros(1,length(wl));
TM_celloutput.TE.AA = zeros(1,length(wl));
%TM_celloutput.mixed.AA = zeros(1,length(wl));
%TM_celloutput.TM.E2 = zeros(length(TMinput.xpos),length(wl));
TM_celloutput.TE.E2 = zeros(length(TMinput.xpos),length(wl));
%TM_celloutput.mixed.E2 = zeros(length(TMinput.xpos),length(wl));
%TM_celloutput.TM.Gen = zeros(length(TMinput.xpos),length(wl));
TM_celloutput.TE.Gen = zeros(length(TMinput.xpos),length(wl));
%TM_celloutput.mixed.Gen = zeros(length(TMinput.xpos),length(wl));
%TM_celloutput.TM.AAlayers = zeros(TMinput.numberoflayers+1,length(wl));
TM_celloutput.TE.AAlayers = zeros(TMinput.numberoflayers+1,length(wl));
%TM_celloutput.mixed.AAlayers = zeros(TMinput.numberoflayers+1,length(wl));
%TM_celloutput.TM.AAfractions = zeros(TMinput.numberoflayers+1,length(wl));
TM_celloutput.TE.AAfractions = zeros(TMinput.numberoflayers+1,length(wl));
%TM_celloutput.mixed.AAfractions = zeros(TMinput.numberoflayers+1,length(wl));
TM_celloutput.TE.AAtotalfractions = zeros(TMinput.numberoflayers+1,1);
%TM_celloutput.TM.AAtotalfractions = zeros(1,TMinput.numberoflayers+1);


for ii= 1:1:length(wl)
    wavelength = wl(ii);
    currentTM = transfermatrixcell(TMinput,wavelength);    
 %   TM_celloutput.TM.RR(ii)=currentTM.RR.TM;
    TM_celloutput.TE.RR(ii)=currentTM.RR.TE;
 %   TM_celloutput.mixed.RR(ii)=currentTM.RR.mixed;
 %   TM_celloutput.TM.TT(ii)=currentTM.TT.TM;
    TM_celloutput.TE.TT(ii)=currentTM.TT.TE;
 %   TM_celloutput.mixed.TT(ii)=currentTM.TT.mixed;
 %   TM_celloutput.TM.E2(:,ii)=currentTM.E2.TM(:,1);
    TM_celloutput.TE.E2(:,ii)=currentTM.E2.TE(:,1);
 %   TM_celloutput.mixed.E2(:,ii)=currentTM.E2.mixed(:,1);
 %   TM_celloutput.TM.Gen(:,ii)=currentTM.Gen.TM(:,1);
    TM_celloutput.TE.Gen(:,ii)=currentTM.Gen.TE(:,1);
 %   TM_celloutput.mixed.Gen(:,ii)=currentTM.Gen.mixed(:,1);
 %   TM_celloutput.TM.AA(ii)=currentTM.AA.TM;
    TM_celloutput.TE.AA(ii)=currentTM.AA.TE;
 %   TM_celloutput.mixed.AA(ii)=currentTM.AA.mixed;
 %   TM_celloutput.TM.AAlayers(:,ii)=currentTM.AAlayers.TM;
    TM_celloutput.TE.AAlayers(:,ii)=currentTM.AAlayers.TE;
 %   TM_celloutput.mixed.AAlayers(:,ii)=currentTM.AAlayers.mixed;
 %   TM_celloutput.TM.AAfractions(:,ii)=currentTM.AAfractions.TM;
    TM_celloutput.TE.AAfractions(:,ii)=currentTM.AAfractions.TE;
 %   TM_celloutput.mixed.AAfractions(:,ii)=currentTM.AAfractions.mixed;
    
  
%      mixed.RR(ii) = currentTM.mixed.RR;
%      mixed.TT(ii) = currentTM.mixed.TT;
%      mixed.AA(ii) = currentTM.mixed.AA;

end

%
TM_cellabsnorm = zeros(1,length(light.wl));
TM_cellabsnorm(1,:) = 1;
  
for ii=1:TMinput.numberoflayers + 1
            TM_celloutput.TE.AAtotalfractions(ii,1) = trapz(light.wl,TM_celloutput.TE.AAfractions(ii,:))/trapz(light.wl,TM_cellabsnorm);
            %TM_cell.AAtotalfractions.TM(1,ii) = trapz(light.wl,TM_cell.AAfractions.TM(ii,1))/trapz(light.wl,TM_cellabsnorm);
end


TM_celloutput.TE.activeGen = sum(TM_celloutput.TE.Gen,2).*(wl(2)-wl(1)); %charge-carriers per m^2, per m, per s.
%TM_celloutput.TM.activeGen = sum(TM_celloutput.TM.Gen,2).*(wl(2)-wl(1)); %charge-carriers per m^2, per m, per s.
%TM_celloutput.mixed.activeGen = (TM_celloutput.TE.activeGen + TM_celloutput.TM.activeGen)./2; %charge-carriers per m^2, per m, per s.
TM_celloutput.TE.Jsc = 1.6e-19*1e-9*sum(TM_celloutput.TE.activeGen)/10; % in mA/cm^2;
%TM_celloutput.TM.Jsc = 1.6e-19*1e-9*sum(TM_celloutput.TM.activeGen)/10; % in mA/cm^2;
%TM_celloutput.mixed.Jsc = (TM_celloutput.TE.Jsc + TM_celloutput.TM.Jsc)./2; % in mA/cm^2;
%TM_celloutput.mixed.Jsc = (light.wl(2) - light.wl(1))*(1/10)*1.6e-19*sum(TM_celloutput.mixed.AAlayers(TMinput.activelayer,:).*TMinput.sunphotons); % in mA/cm^2;


GraphPol = TM_celloutput.TE; %For now! has to be TE and normal incidence. The TM code isn't correct...?
%%
figure;
plot(wl, GraphPol.TT, wl, GraphPol.RR, wl, GraphPol.AA, wl, GraphRTA.TT, wl, GraphRTA.RR, wl, GraphRTA.AA);
makegraphnice();
xlabel('Wavelength (nm)'), ylabel('R,T,A');
xlim([min(wl) max(wl)]); 
%ylim([0 1]);

figure;
plot(TMinput.xpos, GraphPol.activeGen,'.');
makegraphnice();
xlabel('Position in device (nm)'), ylabel('Active generation (/(m^2.m.s))');
xlim([min(TMinput.xpos) max(TMinput.xpos)]); 
%ylim([1 1.1*max(GraphPol.activeGen)]);

figure;
y2 = GraphPol.AAfractions(1,:);
plot(wl, y2);%
hold on
for ii=2:TMinput.numberoflayers
    y2 = y2 + GraphPol.AAfractions(ii,:);
    plot (wl, y2)
end
hold off
makegraphnice();
xlabel('Wavelength (nm)'), ylabel('Absorption fraction');
xlim([min(wl) max(wl)]); 
ylim([0 1]);
%

figure;
y2 = GraphPol.AAlayers(1,:);
plot(wl, y2);%
hold on
for ii=2:TMinput.numberoflayers
    y2 = y2 + GraphPol.AAlayers(ii,:);
    plot (wl, y2)
end
hold off
makegraphnice();
xlabel('Wavelength (nm)'), ylabel('Absorption');
xlim([min(wl) max(wl)]); 
%ylim([0 1]);

figure;
surf(wl,TMinput.xpos,GraphPol.E2, 'linestyle', 'none');
colormap autumn;
makegraphnice()
xlim([min(wl) max(wl)]); 
ylim([0 max(TMinput.xpos)]);
xlabel('\lambda (nm)'),ylabel ('x (nm)'), zlabel ('|E^2|');
%TMcelloutput.TEfield = zeros(length(wl), length(TMinput.xpos));

%figure;
%plot(TMinput.xpos, TM_celloutput.E2.TM(:,2), TMinput.xpos, TM_celloutput.E2.TE(:,2));
%makegraphnice();
%xlabel('x position (nm)'), ylabel('E2 check');
%xlim([400 1200]); 
%ylim([0 1]);


%% TM thickness Jsc scan

scan_input.start = 0;
scan_input.stop = 300;
scan_input.step = 5;
scan_input.layer = 1;

scan_output = TMthicknessJscscan(TMinput,scan_input);
